package javax.swing.text;

import java.awt.Container;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.FontMetrics;

class GlyphPainter1 extends GlyphView.GlyphPainter
{
    FontMetrics metrics;
    
    GlyphPainter1() {
    }
    
    @Override
    public float getSpan(GlyphView glyphView, int n, int n2, TabExpander tabExpander, float n3) {
        this.sync(glyphView);
        Segment text = glyphView.getText(n, n2);
        int tabbedTextWidth = Utilities.getTabbedTextWidth(glyphView, text, this.metrics, (int)n3, tabExpander, n, this.getJustificationData(glyphView));
        SegmentCache.releaseSharedSegment(text);
        return (float)tabbedTextWidth;
    }
    
    @Override
    public float getHeight(GlyphView glyphView) {
        this.sync(glyphView);
        return (float)this.metrics.getHeight();
    }
    
    @Override
    public float getAscent(GlyphView glyphView) {
        this.sync(glyphView);
        return (float)this.metrics.getAscent();
    }
    
    @Override
    public float getDescent(GlyphView glyphView) {
        this.sync(glyphView);
        return (float)this.metrics.getDescent();
    }
    
    @Override
    public void paint(GlyphView glyphView, Graphics graphics, Shape shape, int n, int n2) {
        this.sync(glyphView);
        TabExpander tabExpander = glyphView.getTabExpander();
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        int x = rectangle.x;
        int startOffset = glyphView.getStartOffset();
        int[] justificationData = this.getJustificationData(glyphView);
        if (startOffset != n) {
            Segment text = glyphView.getText(startOffset, n);
            x += Utilities.getTabbedTextWidth(glyphView, text, this.metrics, x, tabExpander, startOffset, justificationData);
            SegmentCache.releaseSharedSegment(text);
        }
        int n3 = rectangle.y + this.metrics.getHeight() - this.metrics.getDescent();
        Segment text2 = glyphView.getText(n, n2);
        graphics.setFont(this.metrics.getFont());
        Utilities.drawTabbedText(glyphView, text2, x, n3, graphics, tabExpander, n, justificationData);
        SegmentCache.releaseSharedSegment(text2);
    }
    
    @Override
    public Shape modelToView(GlyphView glyphView, int n, Position.Bias bias, Shape shape) throws BadLocationException {
        this.sync(glyphView);
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        int startOffset = glyphView.getStartOffset();
        int endOffset = glyphView.getEndOffset();
        TabExpander tabExpander = glyphView.getTabExpander();
        if (n == endOffset) {
            return new Rectangle(rectangle.x + rectangle.width, rectangle.y, 0, this.metrics.getHeight());
        }
        if (n >= startOffset && n <= endOffset) {
            Segment text = glyphView.getText(startOffset, n);
            int tabbedTextWidth = Utilities.getTabbedTextWidth(glyphView, text, this.metrics, rectangle.x, tabExpander, startOffset, this.getJustificationData(glyphView));
            SegmentCache.releaseSharedSegment(text);
            return new Rectangle(rectangle.x + tabbedTextWidth, rectangle.y, 0, this.metrics.getHeight());
        }
        throw new BadLocationException("modelToView - can't convert", endOffset);
    }
    
    @Override
    public int viewToModel(GlyphView glyphView, float n, float n2, Shape shape, Position.Bias[] array) {
        this.sync(glyphView);
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        int startOffset = glyphView.getStartOffset();
        int endOffset = glyphView.getEndOffset();
        TabExpander tabExpander = glyphView.getTabExpander();
        Segment text = glyphView.getText(startOffset, endOffset);
        int tabbedTextOffset = Utilities.getTabbedTextOffset(glyphView, text, this.metrics, rectangle.x, (int)n, tabExpander, startOffset, this.getJustificationData(glyphView));
        SegmentCache.releaseSharedSegment(text);
        int n3 = startOffset + tabbedTextOffset;
        if (n3 == endOffset) {
            --n3;
        }
        array[0] = Position.Bias.Forward;
        return n3;
    }
    
    @Override
    public int getBoundedPosition(GlyphView glyphView, int n, float n2, float n3) {
        this.sync(glyphView);
        TabExpander tabExpander = glyphView.getTabExpander();
        Segment text = glyphView.getText(n, glyphView.getEndOffset());
        int tabbedTextOffset = Utilities.getTabbedTextOffset(glyphView, text, this.metrics, (int)n2, (int)(n2 + n3), tabExpander, n, false, this.getJustificationData(glyphView));
        SegmentCache.releaseSharedSegment(text);
        return n + tabbedTextOffset;
    }
    
    void sync(GlyphView glyphView) {
        Font font = glyphView.getFont();
        if (this.metrics == null || !font.equals(this.metrics.getFont())) {
            Container container = glyphView.getContainer();
            this.metrics = ((container != null) ? container.getFontMetrics(font) : Toolkit.getDefaultToolkit().getFontMetrics(font));
        }
    }
    
    private int[] getJustificationData(GlyphView glyphView) {
        View parent = glyphView.getParent();
        int[] justificationData = null;
        if ((parent instanceof ParagraphView.Row)) {
            justificationData = ((ParagraphView.Row)parent).justificationData;
            return justificationData;
        }
        return justificationData;
    }
}
